@@ -19,7 +19,7 @@ use UUID::Tiny;
 
 use vars qw($VERSION);
 
-$VERSION = '1.16';
+$VERSION = '1.22';
 
 our $DEBUG = 0;
 
@@ -327,8 +327,6 @@ sub _process_request {
             $cookies{ $key } = $val;
         }
         
-        _log( "\n[$$]--> : $req" );
-
         # 
         # read certain length from socket ( as many bytes as content length )
         #
@@ -346,7 +344,6 @@ sub _process_request {
             # TODO - make sure install script makes the directories properly
             my $filename = "$self->{yote_root_dir}/html/" . substr( $path, 4 );
             if ( -e $filename ) {
-                _log( "'$filename' exists" );
                 my @stat = stat $filename;
 
                 my $content_type = $filename =~ /css$/ ? 'text/css' : 'text/html';
@@ -365,7 +362,6 @@ sub _process_request {
                 }
                 close IN;
             } else {
-                _log( "404 file '$filename' not found" );
                 $sock->print( "HTTP/1.1 404 FILE NOT FOUND\n\n" );
             }
             $sock->close;
@@ -377,7 +373,6 @@ sub _process_request {
         # POST /
 
         if ( $verb ne 'POST' ) {
-            _log( "Attempted not-suppored '$verb' request" );
             $sock->print( "HTTP/1.1 400 BAD REQUEST\n\n" );
             $sock->close;
         }
@@ -389,11 +384,10 @@ sub _process_request {
         };
 
         if( ref $@ eq 'HASH' ) {
-            $out_json = to_json( $@ );
+            $out_json = encode_json( $@ );
         } 
         elsif( $@ ) {
-            _log( "ERROR : $@>" );
-            $out_json = to_json( {
+            $out_json = encode_json( {
                 err => $@,
                                  } );
         }
@@ -405,7 +399,6 @@ sub _process_request {
             'Content-Length: ' . bytes::length( $out_json ),
             );
         
-        _log( "<-- 200 OK [$$] ( " . join( ",", @headers ) . " ) ( $out_json )\n" );
         $sock->print( "HTTP/1.1 200 OK\n" . join ("\n", @headers). "\n\n$out_json\n" );
         
         $sock->close;
@@ -414,14 +407,11 @@ sub _process_request {
 
     } #child
 } #_process_request
-
 sub invoke_payload {
     my( $self, $raw_req_data, $file_uploads ) = @_;
 
-    _log( "payload $raw_req_data " );
-
-    my $req_data = from_json( $raw_req_data );
-
+    my $req_data = decode_json( $raw_req_data );
+    
     my( $obj_id, $token, $action, $params ) = @$req_data{ 'i', 't', 'a', 'pl' };
     
     my $server_root = $self->{STORE}->fetch_server_root;
@@ -430,6 +420,7 @@ sub invoke_payload {
 
     my $id_to_last_update_time;
     my $session = $token && $token ne '_' ? $server_root->_fetch_session( $token ) : undef;
+
     if( $session ) {
         $id_to_last_update_time = $session->get__has_ids2times;
     }
@@ -503,9 +494,6 @@ sub invoke_payload {
     my @should_have = ( @{ _unroll_ids( $store, \@out_ids ) } );
     my( @updates, %methods );
     
-    # _log( "out ids ".join(', ',@out_ids) );
-    # _log( "Should have ".join(', ',@should_have) );
-
     #
     # check if existing are in the session
     #
@@ -554,11 +542,9 @@ sub invoke_payload {
                               methods => \%methods,
                             } );
 
-    _log( "out > $out_json" );
-    
     delete $obj->{SESSION};
     $self->{STORE}->stow_all;
-
+    
     return $out_json;
 } #invoke_payload
 
@@ -604,15 +590,6 @@ sub stow_all {
     $self->SUPER::stow_all;
 } #stow_all
 
-sub _stow {
-    my( $self, $obj, $no_timesave ) = @_;
-    $self->SUPER::_stow( $obj );
-
-    unless( $no_timesave ) {
-        my $obj_id = $self->_get_id( $obj );
-    }
-}
-
 sub _last_updated {
     my( $self, $obj_id ) = @_;
     my( $s, $ms ) = @{ $self->{OBJ_UPDATE_DB}->get_record( $obj_id ) };
@@ -644,7 +621,7 @@ sub __transform_params {
     if( ( index( $param, 'v' ) != 0 && index($param, 'f' ) != 0 ) && !$session->get__has_ids2times({})->{$param} ) {
         # obj id given, but the client should not have that id
         if( $param ) {
-            die "Client requested obj with id '$param' which it should not have.";
+            die { err => 'Sync Error', needs_resync => 1 };
         }
         return undef;
     }
@@ -660,24 +637,27 @@ sub _xform_out {
             my( $offset_start, $offset_end ) = ( $1, $2 );
             for( my $i=$offset_start; $i < $offset_end; $i++ ) {
                 my $file = $files->[$i];
-                my( $orig_filename ) = ( $file =~ /([^\/]*)$/ );
-                my( $extension ) = ( $orig_filename =~ /\.([^.\/]+)$/ );
-
-                # TODO - cleanup, maybe use File::Temp or something
-                my $newname = "/tmp/".UUID::Tiny::create_uuid_as_string();
-                open (FILE, ">$newname");
-                while (read ($file, my $Buffer, 1024)) {
-                    print FILE $Buffer;
+                if( $file ) {
+                    my( $orig_filename ) = ( $file =~ /([^\/]*)$/ );
+                    my( $extension ) = ( $orig_filename =~ /\.([^.\/]+)$/ );
+                    
+                    # TODO - cleanup, maybe use File::Temp or something
+                    my $newname = "/tmp/".UUID::Tiny::create_uuid_as_string();
+                    open (FILE, ">$newname");
+                    my $fh = $file->fh;
+                    while (read ($fh, my $Buffer, 1024)) {
+                        print FILE $Buffer;
+                    }
+                    close FILE;
+                    # create yote object here that wraps the file name
+                    return $self->newobj( {
+                        file_path => $newname,
+                        file_extension => $extension,
+                        file_name => $orig_filename,
+                                          } );
                 }
-                close FILE;
-
-                # create yote object here that wraps the file name
-                return $self->newobj( {
-                    file_path => $newname,
-                    file_extension => $extension,
-                    file_name => $orig_filename,
-                                      } );
-            }
+            } #finding the file
+            return undef;
         }
     }
     return $self->SUPER::_xform_out( $val );
@@ -839,14 +819,24 @@ sub _log {
     Yote::Server::_log(shift);
 }
 
+#
+# fetches or creates session which has a _token field
+#
+sub fetch_session {
+    my( $self, $token ) = @_;
+    my $session = $self->_fetch_session( $token ) || $self->_create_session;
+    $self->{SESSION} = $session;
+    $session;
+}
+
 sub _fetch_session {
     my( $self, $token ) = @_;
     
     $self->{STORE}->lock( 'token_mutex' );
     my $slots = $self->get__token_timeslots();
+
     for( my $i=0; $i<@$slots; $i++ ) {
         if( my $session = $slots->[$i]{$token} ) {
-            #if( $i < $#$slots ) {
             if( $i > 0 ) {
                 # make sure this is in the most current 'boat'
                 $slots->[0]{ $token } = $session;
@@ -882,7 +872,6 @@ sub _create_session {
     # A list of slot 'boats' which store token -> ip
     #
     my $slots     = $self->get__token_timeslots();
-
     #
     # a list of times. the list index of these times corresponds
     # to the slot 'boats'
@@ -903,7 +892,8 @@ sub _create_session {
     #
     my $session = $self->{STORE}->newobj( {
         _has_ids2times => {},
-        _token => $token } );
+        _token => $token }, 'Yote::ServerSession' );
+    
     if( $slot_data->[ 0 ] == $current_time_chunk ) {
         $slots->[ 0 ]{ $token } = $session;
     } else {
@@ -922,8 +912,9 @@ sub _create_session {
     $self->{STORE}->_stow( $slots );
     $self->{STORE}->_stow( $slot_data );
     $self->{STORE}->unlock( 'token_mutex' );
-    
-    return $session;
+
+
+    $session;
 
 } #_create_session
 
@@ -954,20 +945,20 @@ sub fetch_app {
     my( $self, $app_name ) = @_;
     my $apps = $self->get__apps;
     my $app  = $apps->{$app_name};
+
     unless( $app ) {
         eval("require $app_name");
         if( $@ ) {
             # TODO - have/use a good logging system with clarity and stuff
             # warnings, errors, etc
-            _log( "App '$app_name' not found $@" );
             return undef;
         }
-        _log( "Loading app '$app_name'" );
         $app = $app_name->_new( $self->{STORE} );
         $apps->{$app_name} = $app;
     }
-    
-    return $app, $self->{SESSION} ? $self->{SESSION}->get_acct : undef;
+    my $acct = $self->{SESSION} ? $self->{SESSION}->get_acct : undef;
+
+    return $app, $acct, $self->{SESSION};
 } #fetch_app
 
 sub fetch_root {
@@ -991,6 +982,30 @@ sub update {
 
 # ------- END Yote::ServerRoot
 
+package Yote::ServerSession;
+
+use base 'Yote::ServerObj';
+
+sub fetch {  # fetch scrambled id
+    my( $self, $in_sess_id ) = @_;
+    $self->get__ids([])->[$in_sess_id];
+}
+
+sub getid { #scramble id for object
+    my( $self, $obj ) = @_;
+    my $o2i = $self->get__obj2id({});
+    if( $o2i->{$obj} ) {
+        return $o2i->{$obj};
+    }
+    my $ids = $self->get__ids([]);
+    push @$ids, $obj;
+    my $id = $#$ids;
+    $o2i->{$obj} = $id;
+    $id;
+} #id
+
+# ------- END Yote::ServerSession
+
 1;
 
 __END__
